body {
	margin: 0px;
	padding: 0px;
	font-family: sans-serif;
	background: white;
	color: black;
	height: 101%;
}

body.with-star {
	background: white url(star.png) -100px -200px no-repeat;
}

#page {
	margin: 0px;
	padding: 0px;
}

div.in-column {
	margin: 0 0 2em 1em;
	float: right;
	max-width: 12em;
}

hr {
	color: #888;
	background: #888;
	border: 0;
	height: 1px;
	width: 90%;
	text-align: center;
	clear: both;
}


div.body {
	clear: both;
}


div.sidebar {
	position: absolute;
	text-align: left;
	right: 0px;
	top: 60px;
	width: 27ex;
	padding-left: 1ex;
	border-left: 1ex solid #eee;
	margin-top: 4em;
}

div.sidebar h2 {
	margin-top: 0;
	padding: 5px 2ex 5px 2ex;
	background: url(t.png) top left repeat-y;
	font-size: 100%;
}

#general {
	list-style: none;
	background: #2E3436 url(general_bg.png) 0 100% repeat-x;
	text-align: right;
	padding: 0 1ex;
	margin: 0;
	font-size: 70%;
}

#general li {
	display: inline;
	background: url(general_separator.png) 0 0 no-repeat;
	padding-top: 10px;
	padding-bottom: 8px;
	margin-left: 0px;
	margin-top: 0px;
}

#general li a {
	font-weight: bold;
	color: #FFFFFF;
	margin: 0 2ex;
	text-decoration: none;
	line-height: 30px;
}

#general li a:hover {
	text-decoration: underline;
}

#general .home {
	float: left;
	background: url(general_separator.png) 100% 0 no-repeat;
	padding-top: 0;
	padding-bottom: 0;
}

#general .home a {
	float: left;
	background: url(foot.png) 7px 50% no-repeat;
	margin-left: 0;
	padding-left: 27px;
}


#header {
	background: #729FCF url(top-header-gradient.png) top left repeat-x;
	float: left;
	width: 100%;
	font-size: 75%;
}

#header-wrap {
	background: transparent url(robot.png) 15px 12px no-repeat;
	min-height: 67px;
}


#header h1 {
	margin: 0;
	margin-left: 85px;
	padding-top: 25px;
	font-size: 200%;
	color: #eeeeec;
}

#header h1 a {
	text-decoration: inherit;
	color: inherit;
}

#tabs {
	background: url(bar.png) 0 100% repeat-x;
	width: 100%;
	float: left;
	margin: 0;
	padding: 0;
}

#portal-globalnav {
	float: right;
	list-style: none;
	margin: 0;
	margin-right: 3ex;
}

#portal-globalnav li {
	float: left;
	margin: 0;
	margin-left: 0.2ex;
	font-size: 2ex;
}

#portal-globalnav li a:hover {
	color: #111111;
}

#portal-globalnav li a {
	float: left;
	text-decoration: none;
	color: #555555;
	background: url(tab_left.png) 0 0 no-repeat;
	padding: 7px 0 7px 7px;
	border-bottom: 2px solid #CCCCCC;
}

#portal-globalnav li span {
	background: url(tab_right.png) 100% 0 no-repeat;
	padding: 7px 28px 7px 19px;
}

#portal-globalnav li.selected a {
	color: #3566A5;
	background: url(tab_left.png) 0 -57px no-repeat;
	border-bottom: none;
	padding-top: 8px;
	padding-bottom: 8px;
}

#portal-globalnav li.selected a span {
	background: url(tab_right.png) 100% -57px no-repeat;
	padding-top: 8px;
	padding-bottom: 8px;
}

div.body {
	background-color: transparent;
	border: none;
	padding: 1em;
} 

div.body-sidebar {
	margin-right: 27ex;
}

a {
	color: #3465a4;
}

div.navbar {
	margin: 0 29ex 0 2ex;
}

a img {
	border: none;
}

div#footer {
	border-top: 1px solid gray;
	color: #888;
	font-size: 80%;
	margin: 3em 3em 1em;
	padding-top: 1.5em;
	text-align: center;
}

/* Buildbot stuff */

table.ProjectSummary tbody th {
	text-align: left;
}

table.ProjectSummary th a {
	font-weight: normal;
	text-decoration: none;
	border-bottom: 1px dotted #3465a4;
}

table.ProjectSummary thead td {
	text-align: center;
}

table.ProjectSummary tbody td {
	border: 1px solid #888;
	padding: 0 3px;
	text-align: center;
}

table.ProjectSummary tbody td.feed {
	border: none;
	padding-right: 2px;
}

th.offline, th.idle, th.building {
	background: transparent url(offline.png) center right no-repeat;
	padding: 0 16px;
	cursor: help;
}

th.idle {
	background-image: url(idle.png);
}

th.building {
	background-image: url(building.png);
}

th.offline {
	color: #888;
}

.success {
	background: #8f8;
}

.failure {
	background: #f88;
}
td.running,
td.building {
	background: #ff8;
}
td.idle {
	color: #888;
	background: #f8f8f8;
}
td.offline {
	color: #000;
	background: #f88;
}
td.failedchecks {
	background: #fa4;
}


table.waterfall {
}

table.waterfall td.Time {
	background: #e0e0e0;
	border: 1px solid #888;
	text-align: center;
	padding: 0 2px;
}

table.waterfall td.LastBuild,
table.waterfall td.BuildStep {
	border: 1px solid #888;
}

table.waterfall .Change {
	padding-right: 1em;
}

table.waterfall td.start {
	background: #ff8;
	font-weight: bold;
}

table.waterfall td.start a {
	text-decoration: none;
}

table.waterfall td.exception {
	border: 3px solid red;
	background: #c66;
}

/* From classic */
/*
td.Change, td.Time, td.Event {
    background: #e0e0e0;
}

td.Event, td.BuildStep, td.Activity, td.Change, td.Time, td.Builder, td.LastBuild {
    border-top: 1px solid;
    border-right: 1px solid;
    border-bottom: 1px solid;
    border-left: 1px solid;
}
*/

tr.totals td {
    padding-top: 1ex;
    text-align: center;
    font-weight: bold;
}

div.changeset ul {
	margin-left: 0em;
	padding-left: 0em;
	list-style: none;
}

div.changeset ul ul {
	padding-left: 1em;
}

ul#tests-summary {
	margin: 1em;
	padding: 0;
	list-style: none;
	-webkit-column-width: 15em;
	-moz-column-width: 15em;
	column-width: 15em;
	-webkit-column-gap: 1em;
	-moz-column-gap: 1em;
	column-gap: 1em;
}

ul#tests-summary li {
	margin-bottom: 1ex;
}

li.changeset {
	border: 1px dashed #555;
	background: #eee;
	padding: 1ex;
	font-size: 80%;
	margin-bottom: 1em;
}

li.changeset p {
	margin: 0;
}

li.changeset pre {
	margin: 0;
}
